<?php

/**
 * Carerix Wordpress Plugin
 *
 * @category template manager
 * @author Bogdan Ivanov
 * @copyright Copyright (c) 2013 Carerix.com (http://www.carerix.com)
 * @link http://www.carerix.com
 * @since Aug 26, 2013 11:36:11 AM
 * @version $Rev$
 */

/**
 * Installation class
 * Used to setup the required tables and default values.
 */
class Environment_Installation {

    const TABLE_SOURCES = 'sources';
    const TABLE_SETTINGS = 'settings';
    const TABLE_FORMS = 'forms';
    const TABLE_BUFFERS = 'buffers';
    const TABLE_POOL = 'syncpool';
    const CARERIX_USER = '_carerix_';

    private $interfacer = null;

    public function __construct() {
        $this->interfacer = WP_Interfacer::getInstance();
    }

	/**
	 * Preparations to do on installation
	 *
	 * @return bool true=first install, false=upgraded
	 */
	public function install() {
        $this->cleanUpUnsecureDirs();
        $this->setDirectories();
        Utils_Cache::cleanEnvironment();
		$settingsTableAlreadyExisting = $this->existsSettingsTable();
        $this->createSettingsTable();
        $this->createSourcesTable();
        $this->createFormsTable();
        $this->createBuffersTable();
        $this->interfacer->user_insert(WP_Constants::CARERIX_USER_NICENAME, self::randomPassword());

		return !$settingsTableAlreadyExisting; // true means the plugin is first installed (instead of upgraded)
    }

	public function firstInstall() {
		Environment_Settings::setAppendLanguageSuffix("off");
	}

	public function upgradeInstall() {
		Entity_Source_Job::upgradeToBodyTexts();
	}

    public function cleanUpUnsecureDirs() {
        try {
            $_wordpressUploadDir = wp_upload_dir();
            $baseDirectory = rtrim($_wordpressUploadDir['basedir'], '/\\');

            $folders = array('carerix_cache', 'carerix_logs', 'carerix_uploads', 'carerix_exports');

            foreach ($folders as $folder) {
                $this->rmdir($baseDirectory . DIRECTORY_SEPARATOR . $folder);
            }
        } catch (Exception $e) {
            
        }
    }

    public function setDirectories(): void {
        $baseDirectory = Carerix::$TMPDIR . DIRECTORY_SEPARATOR;
	    $cacheDirPath               = $baseDirectory . 'cache';
	    $logsPath                   = $baseDirectory . 'logs';
	    $lastSyncDirPath            = $baseDirectory . 'syncstatus';
	    $uploadsPath                = $baseDirectory . 'uploads';
	    $applicationSubmissionsPath = $baseDirectory . 'applicationsubmissionqueue';
	    $exportsPath                = $baseDirectory . 'exports';
        $htaccess = '../.htaccess';
        if (!file_exists($baseDirectory)) {
            mkdir($baseDirectory, 0757, true);
        }
        if (!file_exists($cacheDirPath)) {
            mkdir($cacheDirPath, 0757, true);
        }
        if (!file_exists($logsPath)) {
            mkdir($logsPath, 0757, true);
        }
	    if (!file_exists($lastSyncDirPath)) {
			mkdir($lastSyncDirPath, 0757, true);
	    }
        if (!file_exists($uploadsPath)) {
            mkdir($uploadsPath, 0757, true);
        }
        if (!file_exists($applicationSubmissionsPath)) {
            mkdir($applicationSubmissionsPath, 0757, true);
        }
        if (!file_exists($exportsPath)) {
            mkdir($exportsPath, 0757, true);
        }
        
        if (file_exists($htaccess) && is_writable($htaccess)) {
            @chmod($htaccess, 0755);
        }
    }

    static function get_publications_feed() {
    	$url = self::getAppDomain() . '/' . WP_Constants::FEED_JOBS;
	    if (WP_Constants::M440_DEVMODE == 1) {
	    	$url = "http://cxplugin.carerixsite.nl/cxtools/wp_feed.php";
	    }
        return $url;
    }

    static function get_rss_feed() {
        return self::getAppDomain() . '/' . WP_Constants::FEED_RSS_FEED;
    }

    static function get_candidates_feed() {
        return self::getAppDomain() . '/' . WP_Constants::FEED_CANDIDATES;
    }

    static function get_people_feed() {
        return self::getAppDomain() . '/' . WP_Constants::FEED_PEOPLE;
    }

    static function get_companies_feed() {
        return self::getAppDomain() . '/' . WP_Constants::FEED_COMPANIES;
    }

    static function get_newsletters_feed() {
        return self::getAppDomain() . '/' . WP_Constants::FEED_NEWSLETTERS;
    }

    static function getAppDomain() {
        return str_replace('[appname]', self::getAppName(), WP_Constants::DOMAIN_URL_TEMPLATE);
    }

    static function getAppName() {
        $settingsTable = new WP_Table(Environment_Installation::TABLE_SETTINGS);
        $app_name_rows = $settingsTable->getByKey('app_name');
        if (count($app_name_rows) > 0) {
            return $app_name_rows[0]['value'];
        }

        return '';
    }

    static function getAppToken() {
        $settingsTable = new WP_Table(Environment_Installation::TABLE_SETTINGS);
        $app_token_rows = $settingsTable->getByKey('app_token');
        if (count($app_token_rows) > 0) {
            return $app_token_rows[0]['value'];
        }

        return '';
    }

    static function getAppReporting($asRow = false) {
        $settingsTable = new WP_Table(Environment_Installation::TABLE_SETTINGS);
        $app_reporting_rows = $settingsTable->getByKey('app_reporting');

        if (count($app_reporting_rows) > 0) {
            if ($asRow) {
                return $app_reporting_rows[0];
            }

            if (isset($app_reporting_rows[0])) {
                return $app_reporting_rows[0]['value'];
            }
        }

        return '';
    }

    static function get_table_name($basic_name) {
        $interfacer = WP_Interfacer::getInstance();
        return $interfacer->db_get_prefix() .
                '_' . $interfacer->get_plugin_prefix() .
                '_' . $basic_name;
    }

    private function createSourcesTable() {
        $sourcesTableName = self::get_table_name(Environment_Installation::TABLE_SOURCES);

        $sourcesTableCreate = "CREATE TABLE IF NOT EXISTS `$sourcesTableName` (
			`id` INT NOT NULL AUTO_INCREMENT,
			`type` varchar(255) NOT NULL,
			`name` varchar(255) NOT NULL,
			`settings` text,
			PRIMARY KEY (id)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8";

        $this->interfacer->db_query($sourcesTableCreate);
    }

    private function createSettingsTable() {
        $settingsTableName = self::get_table_name(Environment_Installation::TABLE_SETTINGS);

        $settingsTableCreate = "CREATE TABLE IF NOT EXISTS `$settingsTableName` (
			`id` INT NOT NULL AUTO_INCREMENT,
			`key` varchar(255) NOT NULL,
			`value` varchar(255) NOT NULL,
			PRIMARY KEY (id)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8";

        $this->interfacer->db_query($settingsTableCreate);

        $alter = "ALTER TABLE `$settingsTableName` MODIFY `value` varchar(65536)";
        $this->interfacer->db_query($alter);
    }

	private function existsSettingsTable()
	{
		$settingsTableName = self::get_table_name(Environment_Installation::TABLE_SETTINGS);
		$sql = "SHOW TABLES LIKE '$settingsTableName'";
		$row_count = $this->interfacer->db_query($sql);
		return $row_count > 0;
	}

	private function createFormsTable() {
        $formsTableName = self::get_table_name(Environment_Installation::TABLE_FORMS);

        $formsTableCreate = "CREATE TABLE IF NOT EXISTS `$formsTableName` (
			`id` INT NOT NULL AUTO_INCREMENT,
			`type` varchar(255) NOT NULL,
			`name` varchar(255) NOT NULL,
			`settings` text,
			PRIMARY KEY (id)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8";

        $this->interfacer->db_query($formsTableCreate);

        Environment_Installation::createDefaultForm();
    }

    private static function createDefaultForm() {
        $formsTable = new WP_Table(Environment_Installation::TABLE_FORMS);
        $all_forms = $formsTable->getAll();
        $default_not_found = true;
        foreach ($all_forms as $form) {
            $form_settings = unserialize($form['settings']);

            if (isset($form_settings['is_default'])) {
                $default_not_found = false;
                break;
            }
        }

        if ($default_not_found) {
            $default_form_data = array(
                'post_title' => 'Default Form',
            );

            $default_form_data = array_merge($default_form_data, Entity_Form_Apply::$DEFAULT_SETTINGS);

            $applyForm = new Entity_Form_Apply();
            $applyForm->save($default_form_data);
        }
    }

    public static function getDefaultForm() {
        $formsTable = new WP_Table(Environment_Installation::TABLE_FORMS);
        $all_forms = $formsTable->getAll();

        foreach ($all_forms as $form) {
            $form_settings = unserialize($form['settings']);

            if (isset($form_settings['is_default'])) {
                return $form;
                break;
            }
        }
        return false;
    }

    private function createBuffersTable() {
        $buffersTableName = self::get_table_name(Environment_Installation::TABLE_BUFFERS);

        $buffersTableCreate = "CREATE TABLE IF NOT EXISTS `$buffersTableName` (
			 `id` INT NOT NULL AUTO_INCREMENT,
			 `time` DATETIME,
			 `lastname` varchar(255),
			 `data` text,
			 `error` text,
			 PRIMARY KEY (id)
		 ) ENGINE=InnoDB DEFAULT CHARSET=utf8";

        $this->interfacer->db_query($buffersTableCreate);
    }

    static function randomPassword() {
        $alphabet = 'abcdefghijklmnopqrstuwxyzABCDEFGHIJKLMNOPQRSTUWXYZ0123456789';
        $pass = '';
        $alphaLength = strlen($alphabet) - 1;
        for ($i = 0; $i < 8; $i++) {
            $n = rand(0, $alphaLength);
            $pass .= $alphabet[$n];
        }
        return $pass;
    }

    public static function resetToDefault() {
        Utils_Cache::cleanEnvironment();
        // settings - do not delete credentials
        // sources - do not delete the source with default
        // delete all posts by carerix
        // forms - do not delete the default form
        // buffers - delete all
	    // rest api sync status files - delete all

        $appReporting = Environment_Installation::getAppReporting(true);

        // reset Settings
        // delete app_reporting, but keep app_token and app_name
        if (isset($appReporting['id'])) {
            $settings = new WP_Table(Environment_Installation::TABLE_SETTINGS);
            $settings->delete($appReporting['id']);
        }

        // remove all posts and pages created by _carerix_

        $interface = WP_Interfacer::getInstance();
        $interface->entity_delete_by_author(WP_Constants::CARERIX_USER_NICENAME);
        $interface->entity_delete_by_author(WP_Constants::META_KEY_PREFIX);

        // reset Buffers
        $buffers = Table_Buffer::get();
        $buffers->deleteAll();

        // reset Forms
        $forms = new WP_Table(Environment_Installation::TABLE_FORMS);
        $forms->deleteAll();

        // reset sources
        $sources = Table_Source::get();
        $sources->deleteAll();

        // wipe api syncstatus txt files (wp-content/plugins/CxWordpress/tmp/syncstatus/*.txt)
	    $cronCleaner = new Utils_CronCleaner(0);
	    $cronCleaner->clearRestApiSyncStatuses();

	    // add default form
        Environment_Installation::createDefaultForm();

        // add default job source
        $appname = Environment_Installation::getAppName();
        $apptoken = Environment_Installation::getAppToken();

        if ($appname && $apptoken) {
            Entity_Source_Job::addDefaultJobSource();
//            Entity_Form_Apply::createDefaultPage();
        }
    }

    static public function isSane($detailed = false) {

        $baseDirectory = Carerix::$TMPDIR;

        if (!is_writable($baseDirectory)) {
            return false;
        }

        $toCheck = array('applicationsubmissionqueue', 'cache', 'exports', 'logs', 'syncstatus', 'uploads');

        $isSane = true;
        $notSane = array();
        foreach ($toCheck as $checkable) {
            $path = $baseDirectory . DIRECTORY_SEPARATOR . $checkable;
            $path = str_replace('/', DIRECTORY_SEPARATOR, $path);
            
            if (!file_exists($path)) {
                mkdir($path, 0777, true);
            }
            
            $writable = is_writable($path);
            if (!$writable) {
                array_push($notSane, $path);
            }
            $isSane = $isSane && $writable;
        }
        if ($detailed && !$isSane) {
            return $notSane;
        }
        return $isSane;
    }

	/**
	 * Get max file upload in megabytes or bytes
	 *
	 */
    static public function get_max_file_upload( $as_megabytes_instead_of_bytes = true ) : int {
        // return self::_return_bytes(ini_get('post_max_size')) / 1024 / 1024;
	    $max_filesize = (int) min(self::_convertPHPSizeToBytes(ini_get('post_max_size')), self::_convertPHPSizeToBytes(ini_get('upload_max_filesize'))) / 1024 / 1024;
	    $max_filesize_capped = ( $max_filesize <= WP_Constants::CARERIX_API_MAX_ATTACHMENT_SIZE_MB ) ? $max_filesize : WP_Constants::CARERIX_API_MAX_ATTACHMENT_SIZE_MB;
	    $max_filesize_capped = ($as_megabytes_instead_of_bytes) ? $max_filesize_capped : $max_filesize_capped * 1024 * 1024;
	    return $max_filesize_capped;
    }


	/**
	 * This function transforms the php.ini notation for numbers (like '2M') to an integer (2*1024*1024 in this case)
	 *
	 * @param string $sSize
	 * @return integer The value in bytes
	 * @link https://stackoverflow.com/a/22500394
	 */
	static private function _convertPHPSizeToBytes($sSize)
	{
		//
		$sSuffix = strtoupper(substr($sSize, -1));
		if (!in_array($sSuffix,array('P','T','G','M','K'))){
			return (int)$sSize;
		}
		$iValue = substr($sSize, 0, -1);
		switch ($sSuffix) {
			case 'P':
				$iValue *= 1024;
			// Fallthrough intended
			case 'T':
				$iValue *= 1024;
			// Fallthrough intended
			case 'G':
				$iValue *= 1024;
			// Fallthrough intended
			case 'M':
				$iValue *= 1024;
			// Fallthrough intended
			case 'K':
				$iValue *= 1024;
				break;
		}
		return (int)$iValue;
	}


	/**
	 * @deprecated
	 *
	 * @param $val
	 *
	 * @return int|string
	 */
    static private function _return_bytes($val) {
        $val = trim($val);
        $last = strtolower($val[strlen($val) - 1]);
        switch ($last) {
            case 'g':
                $val *= 1024;
            case 'm':
                $val *= 1024;
            case 'k':
                $val *= 1024;
        }
        return $val;
    }

	/**
	 * Remove empty posts categories if enabled in the plugin settings
	 */
	static public function optionally_cleanup_empty_postjob_categories() {
	    $remove_empty_cats = Environment_Settings::getSyncCleanEmptyCategories();
	    if (!$remove_empty_cats) return;

        $taxonomies = array(
            'category'
        );

        $terms = get_terms($taxonomies, array('hide_empty' => false));

        foreach ($terms as $term) {
            $count = (int) $term->count;
            if ($count < 1) { // only process if there are no other sub-categories
                $taxonomy = $term->taxonomy;
                $term_id = $term->term_id;
	            $managed_by_cx = count(get_term_meta($term->term_id, "managed_by")) ? true : false; // CX cats set by \WP_Interfacer::entity_link_category()
	            if ($managed_by_cx) { // Only delete cats created by CX
		            wp_delete_term($term_id, $taxonomy);
	            }
            }
        }
    }

    private function rmdir($dir) {
        if (is_dir($dir)) {
            $files = array_diff(scandir($dir), array('.', '..'));
            foreach ($files as $file) {
                $path = $dir . DIRECTORY_SEPARATOR . $file;
                (is_dir($path)) ? $this->rmdir($path) : unlink($path);
            }
            return rmdir($dir);
        }
    }

}
